"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthpalClient = void 0;
const axios_1 = require("axios");
const interfaces_1 = require("./interfaces");
class AuthpalClient {
    constructor(clientConfigs) {
        this.resumeAttemted = false;
        this.clientConfigs = clientConfigs;
        if (!this.clientConfigs.userChangesEmitter)
            this.clientConfigs.userChangesEmitter = new interfaces_1.UserChangesEmitter();
        this.clientConfigs.userChangesEmitter.subscribe((changes) => {
            if (!changes.authenticated) {
                this.accessToken = null;
                delete localStorage.APC_ATTEMPT_RESUME;
            }
            else {
                localStorage.APC_ATTEMPT_RESUME = true;
            }
        });
    }
    google(credentials) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                (0, axios_1.default)({
                    method: 'post',
                    url: `${this.clientConfigs.googlePostUrl}`,
                    withCredentials: true,
                    data: credentials,
                })
                    .then(({ data }) => __awaiter(this, void 0, void 0, function* () {
                    this.accessToken = data.accessToken;
                    this.clientConfigs.userChangesEmitter.next({
                        type: 'login',
                        authenticated: true,
                    });
                    resolve();
                }))
                    .catch((error) => {
                    var _a;
                    if (((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.status) === 401) {
                        this.clientConfigs.userChangesEmitter.next({
                            type: 'login',
                            authenticated: false,
                        });
                        reject(error);
                    }
                    else
                        reject(error);
                });
            });
        });
    }
    login(credentials) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                (0, axios_1.default)({
                    method: 'post',
                    url: `${this.clientConfigs.loginPostURL}`,
                    withCredentials: true,
                    data: credentials,
                })
                    .then(({ data }) => __awaiter(this, void 0, void 0, function* () {
                    this.accessToken = data.accessToken;
                    this.clientConfigs.userChangesEmitter.next({
                        type: 'login',
                        authenticated: true,
                    });
                    resolve();
                }))
                    .catch((error) => {
                    var _a;
                    if (((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.status) === 401) {
                        this.clientConfigs.userChangesEmitter.next({
                            type: 'login',
                            authenticated: false,
                        });
                        reject(error);
                    }
                    else
                        reject(error);
                });
            });
        });
    }
    getAuthorizationHeader() {
        return this.accessToken
            ? { Authorization: `Bearer ${this.accessToken}` }
            : null;
    }
    attemptResume() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.resumeAttemted) {
                throw new interfaces_1.LibraryMisusageError(`You attempted to call resume but it's been already called.\n'attemptResume()' should only be called once at the start of your application or on page refresh.`);
            }
            else if (localStorage.APC_ATTEMPT_RESUME === 'true') {
                this.resume();
            }
            else {
                if (this.clientConfigs.resumeDoneMiddleware) {
                    yield this.clientConfigs.resumeDoneMiddleware({
                        type: 'resume',
                        authenticated: false,
                    });
                }
                this.clientConfigs.resumeDoneEmitter.complete();
            }
        });
    }
    resume() {
        return __awaiter(this, void 0, void 0, function* () {
            (0, axios_1.default)({
                method: 'get',
                url: `${this.clientConfigs.resumeGetURL}`,
                headers: {
                    'Access-Control-Expose-Headers': 'Set-Cookie',
                },
                withCredentials: true,
            })
                .then(({ data }) => __awaiter(this, void 0, void 0, function* () {
                this.accessToken = data.accessToken;
                let changes = {
                    type: 'resume',
                    authenticated: true,
                };
                this.clientConfigs.userChangesEmitter.next(changes);
                if (this.clientConfigs.resumeDoneMiddleware) {
                    yield this.clientConfigs.resumeDoneMiddleware(changes);
                }
                this.clientConfigs.resumeDoneEmitter.complete();
            }))
                .catch((error) => __awaiter(this, void 0, void 0, function* () {
                var _a;
                if (((_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.status) === 401) {
                    let changes = {
                        type: 'resume',
                        authenticated: false,
                    };
                    yield this.clientConfigs.userChangesEmitter.next(changes);
                    if (this.clientConfigs.resumeDoneMiddleware) {
                        yield this.clientConfigs.resumeDoneMiddleware(changes);
                    }
                    this.clientConfigs.resumeDoneEmitter.complete();
                }
                else
                    console.error(error);
            }));
        });
    }
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            (0, axios_1.default)({
                method: 'get',
                url: `${this.clientConfigs.logoutGetURL}`,
                headers: Object.assign({ 'Access-Control-Expose-Headers': 'Set-Cookie' }, this.getAuthorizationHeader()),
                withCredentials: true,
            }).then(() => {
                this.clientConfigs.userChangesEmitter.next({
                    type: 'logout',
                    authenticated: false,
                });
            });
        });
    }
}
exports.AuthpalClient = AuthpalClient;
